

(() => {
    const pluginName = "plugin_EffectSprites";

    const _Sprite_initialize = Sprite.prototype.initialize;
    Sprite.prototype.initialize = function() {
        _Sprite_initialize.apply(this, arguments);
        this._tweens = [];
        this._blink_count = 0;
        this._blink_flag = false;
        this._blink_opacity = 255;
    };

    const _Sprite_update = Sprite.prototype.update;
    Sprite.prototype.update = function() {
        _Sprite_update.apply(this, arguments);
        this.update_tweens();
        this.updateBlink();
    };

    Sprite.prototype.update_tweens = function() {
        for (const tween of this._tweens) {
            this.update_tween(tween);
        }
        for (var i = this._tweens.length - 1; i >= 0; i--) {
            if (this._tweens[i].effect_count < 0) {
                this._tweens.splice(i, 1);
            }
        }
    };

    Sprite.prototype.update_tween = function(tween) {
        if (tween.delay > 0) {
            tween.delay -= 1;
            return;
        }
        switch (tween.effect_type) {
            case "opacity":
                this.opacity = Math.floor(((tween.end_opacity) * (tween.time - tween.effect_count) + (tween.start_opacity) * (tween.effect_count)) / tween.time);
                break;
            case "blend_color":
                const alpha = 128 - (tween.time - tween.effect_count) * (128 / tween.time);
                this.setBlendColor([tween.color[0], tween.color[1], tween.color[2], alpha]);
                break;
            case "positions":
                var pos = tween.positions[tween.time - tween.effect_count];
                this.x = pos.x;
                this.y = pos.y;
                break;
            case "move":
                this.x = Math.floor(((tween.end_position[0]) * (tween.time - tween.effect_count) + (tween.start_position[0]) * (tween.effect_count)) / tween.time);
                this.y = Math.floor(((tween.end_position[1]) * (tween.time - tween.effect_count) + (tween.start_position[1]) * (tween.effect_count)) / tween.time);
                break;
            case "scale":
                this.scale.x = (((tween.end_scale[0]) * (tween.time - tween.effect_count) + (tween.start_scale[0]) * (tween.effect_count)) / tween.time);
                this.scale.y = (((tween.end_scale[1]) * (tween.time - tween.effect_count) + (tween.start_scale[1]) * (tween.effect_count)) / tween.time);
                break;
            case "rotation":
                this.rotation = (((tween.end_rotation) * (tween.time - tween.effect_count) + (tween.start_rotation) * (tween.effect_count)) / tween.time);
                break;
        }
        tween.effect_count -= 1;
    };

    Sprite.prototype.startOpacity = function(_time, _start_opacity, _end_opacity, _delay) {
        this._tweens.push({ "effect_type" : "opacity", "start_opacity" : _start_opacity, "end_opacity" : _end_opacity, "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startAppear = function() {
        this.startOpacity(16, 0, 255, 0);
    };

    Sprite.prototype.startDisappear = function() {
        this.startOpacity(32, 255, 0, 0);
    };

    Sprite.prototype.startBlendColor = function(_time, _color, _delay) {
        this._tweens.push({ "effect_type" : "blend_color", "color" : _color, "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startWhiten = function() {
        this.startBlendColor(32, [255, 255, 255], 0);
    };

    Sprite.prototype.startPositions = function(_time, _positions, _delay) {
        this._tweens.push({ "effect_type" : "positions", "positions" : _positions, "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startMove = function(_time, _start_x, _start_y, _end_x, _end_y, _delay) {
        this._tweens.push({ "effect_type" : "move", "start_position" : [_start_x, _start_y], "end_position" : [_end_x, _end_y], "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startScale = function(_time, _start_x, _start_y, _end_x, _end_y, _delay) {
        this._tweens.push({ "effect_type" : "scale", "start_scale" : [_start_x, _start_y], "end_scale" : [_end_x, _end_y], "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startRotation = function(_time, _start_rotation, _end_rotation, _delay) {
        this._tweens.push({ "effect_type" : "rotation", "start_rotation" : _start_rotation, "end_rotation" : _end_rotation, "time" : _time, "effect_count" : _time, "delay" : _delay });
    };

    Sprite.prototype.startTransform = function(_time, _end_position, _end_scale, _end_rotation, _end_opacity, _delay) {
        this.startMove(_time, this.x, this.y, _end_position[0], _end_position[1], _delay);
        this.startScale(_time, this.scale.x, this.scale.y, _end_scale[0], _end_scale[1], _delay);
        this.startRotation(_time, this.rotation, _end_rotation, _delay);
        this.startOpacity(_time, this.opacity, _end_opacity, _delay);
    };

    Sprite.prototype.startBlink = function() {
        this._blink_flag = true;
        this._blink_opacity = this.opacity;
    };

    Sprite.prototype.updateBlink = function() {
        if (this._blink_flag) {
            this._blink_count += 1;
            this.opacity = this._blink_count % 10 < 5 ? 255 : 0;
        }
    };

    Sprite.prototype.endBlink = function() {
        this._blink_flag = false;
        this.opacity = this._blink_opacity;
    };

    Sprite.prototype.isEffecting = function() {
        return this._tweens.length > 0;
    };

})();
