(() => {
  const pluginName = "plugin_ExTitle";

  //-----------------------------------------------------------------------------
  // Scene_OexTitle
  //
  // The scene class of the title screen.

  function Scene_OexTitle() {
    this.initialize(...arguments);
  }

  Scene_OexTitle.prototype = Object.create(Scene_Base.prototype);
  Scene_OexTitle.prototype.constructor = Scene_OexTitle;

  Scene_OexTitle.prototype.initialize = function () {
    Scene_Base.prototype.initialize.call(this);
    this._buttonIndex = 0;
  };

  Scene_OexTitle.prototype.create = function () {
    Scene_Base.prototype.create.call(this);
    this.createBackground();
    this.createWindowLayer();
    this.createButtons();
  };

  Scene_OexTitle.prototype.start = function () {
    Scene_Base.prototype.start.call(this);
    SceneManager.clearStack();
    this.adjustBackground();
    this.playTitleMusic();
    this.startFadeIn(this.fadeSpeed(), false);
  };

  Scene_OexTitle.prototype.update = function () {
    Scene_Base.prototype.update.call(this);
    if (Input.isTriggered("down")) {
      if (this._buttonIndex < 4) {
        this._buttonIndex += 1;
      }
      SoundManager.playCursor();
    }
    if (Input.isTriggered("up")) {
      if (this._buttonIndex > 0) {
        this._buttonIndex -= 1;
      }
      SoundManager.playCursor();
    }
    if (Input.isTriggered("ok")) {
      this.pushButton(this._buttonIndex);
    }
    for (let i = 0; i < 5; i++) {
      var sp = this._title_buttons[i];
      if (i == this._buttonIndex) {
        sp.scale.x = 7.0 / 6.0;
        sp.scale.y = 11.0 / 10.0;
      } else {
        sp.scale.x = 1.0;
        sp.scale.y = 1.0;
      }
    }
  };

  Scene_OexTitle.prototype.isBusy = function () {
    return Scene_Base.prototype.isBusy.call(this);
  };

  Scene_OexTitle.prototype.terminate = function () {
    Scene_Base.prototype.terminate.call(this);
    SceneManager.snapForBackground();
  };

  Scene_OexTitle.prototype.createBackground = function () {
    this._backSprite1 = new Sprite(ImageManager.loadTitle1($dataSystem.title1Name));
    this._backSprite2 = new Sprite(ImageManager.loadTitle2($dataSystem.title2Name));
    this.addChild(this._backSprite1);
    this.addChild(this._backSprite2);
  };

  Scene_OexTitle.prototype.adjustBackground = function () {
    this.scaleSprite(this._backSprite1);
    this.scaleSprite(this._backSprite2);
    this.centerSprite(this._backSprite1);
    this.centerSprite(this._backSprite2);
  };

  Scene_OexTitle.prototype.commandExtra = function () {
    alert("Not available for selection in this demo game.");
  };

  Scene_OexTitle.prototype.commandNewGame = function () {
    DataManager.setupNewGame();
    this.fadeOutAll();
    SceneManager.goto(Scene_Map);
  };

  Scene_OexTitle.prototype.commandContinue = function () {
    SceneManager.push(Scene_Load);
  };

  Scene_OexTitle.prototype.commandOptions = function () {
    SceneManager.push(Scene_Options);
  };

  Scene_OexTitle.prototype.playTitleMusic = function () {
    AudioManager.playBgm($dataSystem.titleBgm);
    AudioManager.stopBgs();
    AudioManager.stopMe();
  };

  Scene_OexTitle.prototype.createButtons = function () {
    var texts = ["New Game", "Continue", "Extra", "Config", "Exit"];
    this._title_buttons = [];
    for (let i = 0; i < 5; i++) {
      const ci = i;
      var sp = new OexSprite_CommonButton("title_buttons", new Rectangle(0, 0, 400, 60), new Rectangle(0, 0, 400, 60));
      sp.anchor.x = 0.5;
      sp.anchor.y = 0.5;
      sp.x = 500 + 200;
      sp.y = 350 + 80 * i;
      sp.setClickHandler(this.pushButton.bind(this, ci));
      this.addChild(sp);
      this._title_buttons.push(sp);
      var sp2 = new Sprite();
      sp2.bitmap = new Bitmap(400, 60);
      sp2.bitmap.fontFace = $gameSystem.mainFontFace();
      sp2.bitmap.outlineColor = "black";
      sp2.bitmap.outlineWidth = 4;
      sp2.bitmap.fontSize = 32;
      sp2.bitmap.drawText(texts[i], 0, 12, 400, 32, "center");
      sp2.anchor.x = 0.5;
      sp2.anchor.y = 0.5;
      sp2.x = 500 + 200;
      sp2.y = 350 + 80 * i;
      this.addChild(sp2);
    }
  };

  Scene_OexTitle.prototype.pushButton = function (_index) {
    if (this._buttonIndex == _index) {
      switch (this._buttonIndex) {
        case 0:
          //alert("new game");
          OexUtils.playSound("Decision5");
          this.commandNewGame();
          break;
        case 1:
          OexUtils.playSound("Decision2");
          this.commandContinue();
          //alert("load");
          break;
        case 2:
          OexUtils.playSound("Decision2");
          this.commandExtra();
          break;
        case 3:
          OexUtils.playSound("Decision2");
          this.commandOptions();
          //alert("config");
          break;
        case 4:
          SceneManager.exit();
          //alert("exit");
          break;
      }
    }
    this._buttonIndex = _index;
    OexUtils.playSound("__button45");
  };

  Scene_Title = Scene_OexTitle;
})();
