

(() => {
    const pluginName = "plugin_Menu";

    //-----------------------------------------------------------------------------
    // OexSprite_MenuButton
    //
    // The sprite for displaying a button.

    function OexSprite_MenuButton() {
        this.initialize(...arguments);
    }

    OexSprite_MenuButton.prototype = Object.create(Sprite_Clickable.prototype);
    OexSprite_MenuButton.prototype.constructor = OexSprite_MenuButton;

    OexSprite_MenuButton.prototype.initialize = function(buttonType, pos) {
        Sprite_Clickable.prototype.initialize.call(this);
        this._buttonType = buttonType;
        this._pos = pos;
        this._clickHandler = null;
        this.initMembers();
    };

    OexSprite_MenuButton.prototype.initMembers = function() {
        this.createMainSprite();
        this.x = Graphics.boxWidth - (100 * (2 + this._pos)) - 4;
    };

    OexSprite_MenuButton.prototype.createMainSprite = function() {
        this.bitmap = ImageManager.loadPicture("manu_button00");
        this.setFrame(96 * this._buttonType, 0, 96, 48);
    };

    OexSprite_MenuButton.prototype.update = function() {
        Sprite_Clickable.prototype.update.call(this);
        this.updateOpacity();
        this.processTouch();
    };

    OexSprite_MenuButton.prototype.updateOpacity = function() {
        this.opacity = this._pressed ? 255 : 192;
    };

    OexSprite_MenuButton.prototype.setClickHandler = function(method) {
        this._clickHandler = method;
    };

    OexSprite_MenuButton.prototype.onClick = function() {
        if (this._clickHandler) {
            this._clickHandler();
        }
    };

    //-----------------------------------------------------------------------------
    // OexWindow_Item
    //
    // The window for displaying the description of the selected item.

    function OexWindow_Item() {
        this.initialize(...arguments);
    }

    OexWindow_Item.prototype = Object.create(Window_Base.prototype);
    OexWindow_Item.prototype.constructor = OexWindow_Item;

    OexWindow_Item.prototype.initialize = function(rect) {
        Window_Base.prototype.initialize.call(this, rect);
        this._item = null;
    };

    OexWindow_Item.prototype.clear = function() {
        this.contents.clear();
    };

    OexWindow_Item.prototype.setItem = function(item) {
        //this.setText(item ? item.description : "");
        this._item = item;
        this.refresh();
    };

    OexWindow_Item.prototype.refresh = function() {
        //const rect = this.baseTextRect();
        this.contents.clear();
        if (this._item) {
            const bitmap = ImageManager.loadPicture(this._item.meta.thumbnail);
            this.contents.blt(bitmap, 0, 0, 500, 500, 0, 0);
        }
    };

    //-----------------------------------------------------------------------------
    // Scene_OexItem
    //
    // The superclass of Scene_Item and Scene_Skill.

    function Scene_OexItem() {
        this.initialize(...arguments);
    }

    Scene_OexItem.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_OexItem.prototype.constructor = Scene_OexItem;

    Scene_OexItem.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
        this._menu_index = 0;
    };

    Scene_OexItem.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createHelpWindow();
        this.createItemWindow();
        this.createSubButton();
        this.preloadImages();
        this._menu_index = 0;
    };

    Scene_OexItem.prototype.preloadImages = function() {
        var fnames = [
                "item_digit00",
                "item_dryer00",
                "item_heart_key00",
                "item_key00",
                "item_paper01",
                "item_paper02",
                "item_paper03",
                "item_print00",
                "item_print01",
                "item_taimatsu00"];
        for (const img of fnames) {
            ImageManager.loadPicture(img);
        }
    };

    Scene_OexItem.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        if ($gameSwitches.value(5)) {
            this._subButton6.visible = true;
            this._subButton7.visible = false;
            this._subButton5.visible = true;
        } else {
            this._subButton6.visible = false;
            this._subButton7.visible = true;
            this._subButton5.visible = false;
        }
        if (!this._itemWindow.active) {
            if (Input.isTriggered("left")) {
                if (this._menu_index < 6) {
                    this._menu_index += 1;
                }
                SoundManager.playCursor();
            }
            if (Input.isTriggered("right")) {
                if (this._menu_index > 0) {
                    this._menu_index -= 1;
                }
                SoundManager.playCursor();
            }
            if (Input.isTriggered("down")) {
                this._itemWindow.activate();
                SoundManager.playCursor();
            }
            if (Input.isRepeated("cancel") || TouchInput.isCancelled()) {
                SoundManager.playCancel();
                this.popScene();
            }
            if ($gameSwitches.value(5)) {
                this._menu_index = (this._menu_index + 7) % 7;
            } else {
                this._menu_index = (this._menu_index + 6) % 6;
            }
            if (Input.isTriggered("ok")) {
                switch (this._menu_index) {
                  case 0:
                    SoundManager.playCancel();
                    this.popScene();
                    break;
                  case 1:
                    this._subButton1.onClick();
                    break;
                  case 2:
                    this._subButton2.onClick();
                    break;
                  case 3:
                    this._subButton3.onClick();
                    break;
                  case 4:
                    this._subButton4.onClick();
                    break;
                  case 5:
                    $gameSwitches.setValue(5, !$gameSwitches.value(5));
                    break;
                  case 6:
                    this._subButton5.onClick();
                    break;
                }
            }
            this.finger_sp.x = Graphics.boxWidth - (100 * (1 + this._menu_index)) - 4 + 1;
            this.finger_sp.y = this.buttonY() + 1;
            this.finger_sp.visible = true;
        } else {
            if (this._itemWindow.maxItems() <= 0) {
                this._itemWindow.deactivate();
            }
            this.finger_sp.visible = false;
        }
    };

    Scene_OexItem.prototype.createSubButton = function() {
        this._subButton1 = new OexSprite_MenuButton(0, 0);
        this._subButton1.y = this.buttonY();
        this.addWindow(this._subButton1);
        this._subButton1.setClickHandler(this.commandSave.bind(this));
        this._subButton2 = new OexSprite_MenuButton(1, 1);
        this._subButton2.y = this.buttonY();
        this.addWindow(this._subButton2);
        this._subButton2.setClickHandler(this.commandOptions.bind(this));
        this._subButton3 = new OexSprite_MenuButton(2, 2);
        this._subButton3.y = this.buttonY();
        this.addWindow(this._subButton3);
        this._subButton3.setClickHandler(this.commandThink.bind(this));
        this._subButton4 = new OexSprite_MenuButton(4, 3);
        this._subButton4.y = this.buttonY();
        this.addWindow(this._subButton4);
        this._subButton4.setClickHandler(this.commandOperation.bind(this));
        this._subButton5 = new OexSprite_MenuButton(3, 5);
        this._subButton5.y = this.buttonY();
        this.addWindow(this._subButton5);
        this._subButton5.setClickHandler(this.commandNyo10.bind(this));
        this._subButton6 = new OexSprite_MenuButton(5, 4);
        this._subButton6.y = this.buttonY();
        this.addWindow(this._subButton6);
        this._subButton6.setClickHandler(this.commandCheatOff.bind(this));
        this._subButton7 = new OexSprite_MenuButton(6, 4);
        this._subButton7.y = this.buttonY();
        this.addWindow(this._subButton7);
        this._subButton7.setClickHandler(this.commandCheatOn.bind(this));
        this.finger_sp = new Sprite();
        this.finger_sp.bitmap = ImageManager.loadPicture("menu_cursour00");
        this.addChild(this.finger_sp);
    };

    Scene_OexItem.prototype.createHelpWindow = function() {
        const rect = this.helpWindowRect();
        this._helpWindow = new OexWindow_Item(rect);
        this.addWindow(this._helpWindow);
    };

    Scene_OexItem.prototype.helpWindowRect = function() {
        const wx = this.itemWindowWidth();
        const wy = this.mainAreaTop();//this.mainAreaBottom() - this.calcWindowHeight(1, true);
        const ww = Graphics.boxWidth - this.itemWindowWidth();
        const wh = this.mainAreaBottom() - this.mainAreaTop();
        //console.log("ww:" + ww); 635
        //console.log("wh:" + wh); 564
        return new Rectangle(wx, wy, ww, wh);
    };

    Scene_OexItem.prototype.createItemWindow = function() {
        const rect = this.itemWindowRect();
        this._itemWindow = new Window_ItemList(rect);
        this._itemWindow.setHelpWindow(this._helpWindow);
        this._itemWindow.setHandler("ok", this.onItemOk.bind(this));
        this._itemWindow.setHandler("cancel", this.onItemCancel.bind(this));
        this.addWindow(this._itemWindow);
        this._itemWindow.setCategory("keyItem");
        this._itemWindow.activate();
        if (this._itemWindow.maxItems() > 0) {
            this._itemWindow.select(0);
        }
    };

    Scene_OexItem.prototype.itemWindowRect = function() {
        const wx = 0;
        const wy = this.mainAreaTop();
        const ww = this.itemWindowWidth();
        const wh = this.mainAreaBottom() - this.mainAreaTop();
        return new Rectangle(wx, wy, ww, wh);
    };

    Scene_OexItem.prototype.itemWindowWidth = function() {
        return Math.floor(Graphics.boxWidth / 3);
    };

    Scene_OexItem.prototype.item = function() {
        return this._itemWindow.item();
    };

    Scene_OexItem.prototype.onCategoryOk = function() {
        //this._itemWindow.activate();
        //this._itemWindow.selectLast();
    };

    Scene_OexItem.prototype.onItemOk = function() {
        // $gameParty.setLastItem(this.item());
        // this.determineItem();
        this._itemWindow.activate();
    };

    Scene_OexItem.prototype.onItemCancel = function() {
        // if (this._categoryWindow.needsSelection()) {
        //     this._itemWindow.deselect();
        //     this._categoryWindow.activate();
        // } else {
        //     this.popScene();
        // }
        this.popScene();
    };

    Scene_OexItem.prototype.commandThink = function() {
        $gameSwitches.setValue(19, true);
        this.popScene();
    };

    Scene_OexItem.prototype.commandOperation = function() {
        $gameSwitches.setValue(20, true);
        this.popScene();
    };

    Scene_OexItem.prototype.commandCheatOn = function() {
        $gameSwitches.setValue(5, true);
    };

    Scene_OexItem.prototype.commandCheatOff = function() {
        $gameSwitches.setValue(5, false);
    };

    Scene_OexItem.prototype.commandNyo10 = function() {
        var oldValue = $gameVariables.value(4);
        $gameVariables.setValue(4, oldValue + 700);
        OexUtils.playSound("2k-pocha09");
        this.popScene();
    };

    Scene_OexItem.prototype.commandOptions = function() {
        SceneManager.push(Scene_Options);
    };

    Scene_OexItem.prototype.commandSave = function() {
        SceneManager.push(Scene_Save);
    };


    Window_ItemList.prototype.maxCols = function() {
        return 1;
    };

    Window_ItemList.prototype.cursorUp = function(wrap) {
        if (this.index() <= 0) {
            this.deactivate();
        } else {
            Window_Selectable.prototype.cursorUp.apply(this, arguments);
        }
    };

    Scene_Menu = Scene_OexItem;

})();
