function Oex_Sprite_NyoGauge() {
  this.initialize(...arguments);
}

(() => {
  const pluginName = "plugin_uis";

  Input.keyMapper = {
    9: "tab", // tab
    13: "ok", // enter
    16: "shift", // shift
    17: "control", // control
    18: "control", // alt
    27: "escape", // escape
    32: "ok", // space
    33: "pageup", // pageup
    34: "pagedown", // pagedown
    37: "left", // left arrow
    38: "up", // up arrow
    39: "right", // right arrow
    40: "down", // down arrow
    45: "escape", // insert
    65: "left", // A
    68: "right", // D
    69: "pagedown", // E
    81: "pageup", // Q
    83: "down", // S
    87: "up", // W
    88: "escape", // X
    90: "ok", // Z
    96: "escape", // numpad 0
    98: "down", // numpad 2
    100: "left", // numpad 4
    102: "right", // numpad 6
    104: "up", // numpad 8
    120: "debug", // F9
  };

  //-----------------------------------------------------------------------------
  // OexFPSKeeper
  //

  function OexFPSKeeper() {
    this.initialize(...arguments);
  }

  OexFPSKeeper.prototype = Object.create(Sprite.prototype);
  OexFPSKeeper.prototype.constructor = OexFPSKeeper;

  OexFPSKeeper.prototype.initialize = function (_fps) {
    Sprite.prototype.initialize.call(this);
    this._accumulator = 0;
    this._fps = _fps;
    this._deltaTime = 1.0 / this._fps;
    this._currentTime = performance.now();
  };

  OexFPSKeeper.prototype.check = function () {
    var newTime = performance.now();
    var fTime = (newTime - this._currentTime) / 1000;
    if (fTime > 0.25) fTime = 0.25;
    this._currentTime = newTime;
    this._accumulator += fTime;
    if (this._accumulator >= this._deltaTime) {
      this._accumulator -= this._deltaTime;
      return true;
    }
    return false;
  };

  //---------------------------------------------------------------------------------------------------

  //SceneManager.determineRepeatNumber = function(deltaTime) {
  //    if (this._fpsKeeper) {
  //        if (this._fpsKeeper.check()) {
  //            return 1;
  //        }
  //    } else {
  //        this._fpsKeeper = new OexFPSKeeper(60);
  //    }
  //    return 0;
  //};
  // Graphics.startGameLoop = function() {
  //     if (this._app) {
  //         this._app.start();
  //         this._app.ticker.maxFPS = 60;
  //     }
  // };

  Window_Message.prototype.updateMessage = function () {
    const textState = this._textState;
    if (textState) {
      while (!this.isEndOfText(textState)) {
        if (this.needsNewPage(textState)) {
          this.newPage(textState);
        }
        this.updateShowFast();
        if (this._textState.x >= this.contents.width - 32) {
          this.processNewLine(this._textState);
          if (this._textState.text[this._textState.index] == "\n") {
            this._textState.index++;
          }
        }
        this.processCharacter(textState);
        if (this.shouldBreakHere(textState)) {
          break;
        }
      }
      this.flushTextState(textState);
      if (this.isEndOfText(textState) && !this.isWaiting()) {
        this.onEndOfText();
      }
      return true;
    } else {
      return false;
    }
  };
  //-----------------------------------------------------------------------------
  // Show Picture
  Game_Interpreter.prototype.command231 = function (params) {
    const point = this.picturePoint(params);
    // prettier-ignore
    $gameScreen.showPicture(
            params[0], params[1], params[2], point.x, point.y,
            params[6], params[7], params[8], params[9]
        );
    DataManager.addSystemdInfo(params[1]);
    return true;
  };

  //-----------------------------------------------------------------------------
  // OexSprite_SideChar
  //

  function OexSprite_SideChar() {
    this.initialize(...arguments);
  }

  OexSprite_SideChar.prototype = Object.create(Sprite.prototype);
  OexSprite_SideChar.prototype.constructor = OexSprite_SideChar;

  OexSprite_SideChar.prototype.initialize = function () {
    Sprite.prototype.initialize.call(this);
    this.x = -50;
    this.y = 720;
    this.z = 7;
    this.visible = true;
    this._picture_name = "";
    this._pictureSprite = new Sprite();
    this._pictureSprite.anchor.x = 0;
    this._pictureSprite.anchor.y = 1;
    this.addChild(this._pictureSprite);
    this._eyeSprite = new Sprite_Clickable();
    this._eyeSprite.bitmap = ImageManager.loadPicture("eye_icon");
    this._eyeSprite.setFrame(0, 0, 128, 32);
    this._eyeSprite.x = 50;
    this._eyeSprite.y = -500;
    this._eyeSprite.onClick = this.clickEye.bind(this);
    this.addChild(this._eyeSprite);
  };

  OexSprite_SideChar.prototype.update = function () {
    Sprite.prototype.update.call(this);
    this.updateVisible();
    this.updateSwitches();
    if (this._eyeSprite.visible) {
      if (Input.isTriggered("pageup")) {
        this.clickEye();
      }
    }
    if (this._pictureSprite.visible) {
      if ($gameSwitches.value(17)) {
        if (this._picture_name != "h1_011") {
          this._picture_name = "h1_011";
          this._pictureSprite.bitmap = ImageManager.loadPicture(this._picture_name);
          DataManager.addSystemdInfo("h1_011");
        }
      } else if ($gameSwitches.value(16)) {
        if (this._picture_name != "h1_010") {
          this._picture_name = "h1_010";
          this._pictureSprite.bitmap = ImageManager.loadPicture(this._picture_name);
          DataManager.addSystemdInfo("h1_010");
        }
      } else {
        if (this._picture_name != "h1_01") {
          this._picture_name = "h1_01";
          this._pictureSprite.bitmap = ImageManager.loadPicture(this._picture_name);
          DataManager.addSystemdInfo("h1_01");
        }
      }
    }
  };

  OexSprite_SideChar.prototype.updateSwitches = function () {
    if ($gameMap.isEventRunning()) {
    } else {
      var parcent = Math.floor($gameVariables.value(4) / 70);
      if (parcent <= 33) {
      } else if (parcent <= 66) {
        if (!$gameSwitches.value(16)) {
          $gameSwitches.setValue(14, true);
        }
      } else if (parcent <= 99) {
        if (!$gameSwitches.value(17)) {
          $gameSwitches.setValue(15, true);
        }
      } else {
        //if (!$gameSwitches.value(18)) {
        //$gameSwitches.setValue(18, true);
        //}
      }
    }
  };

  OexSprite_SideChar.prototype.updateVisible = function () {
    if ($gameMap.isEventRunning()) {
      this._pictureSprite.visible = false;
    } else if ($gameVariables.value(11) > 0 || $gameVariables.value(13) > 0) {
      this._pictureSprite.visible = false;
    } else {
      this._pictureSprite.visible = true;
    }
    if (!$gameSwitches.value(11)) {
      this._pictureSprite.visible = false;
    }
    this._eyeSprite.visible = this._pictureSprite.visible;
    if (!$gameSwitches.value(10)) {
      this._pictureSprite.visible = false;
    }
  }; // canMove

  OexSprite_SideChar.prototype.clickEye = function () {
    if ($gameSwitches.value(10)) {
      $gameSwitches.setValue(10, false);
      this._eyeSprite.setFrame(0, 32, 128, 32);
    } else {
      $gameSwitches.setValue(10, true);
      this._eyeSprite.setFrame(0, 0, 128, 32);
    }
  }; // canMove

  //-----------------------------------------------------------------------------
  // Oex_Sprite_NyoGauge

  Oex_Sprite_NyoGauge.prototype = Object.create(Sprite_Clickable.prototype);
  Oex_Sprite_NyoGauge.prototype.constructor = Oex_Sprite_NyoGauge;

  Oex_Sprite_NyoGauge.prototype.initialize = function () {
    Sprite_Clickable.prototype.initialize.call(this);
    this.initMembers();
  };

  Oex_Sprite_NyoGauge.prototype.initMembers = function () {
    this.createGaugeSprite();
    this.createMainSprite();
    this.createLabelSprite();
    this.x = 200;
    this.y = 0; //768 - 40 - 8;
  };

  Oex_Sprite_NyoGauge.prototype.createMainSprite = function () {
    this._mainSprite = new Sprite();
    this._mainSprite.anchor.x = 0.0;
    this._mainSprite.anchor.y = 0.0;
    this._mainSprite.bitmap = ImageManager.loadPicture("nyo_gauge_frame00");
    this.addChild(this._mainSprite);
  };

  Oex_Sprite_NyoGauge.prototype.createGaugeSprite = function () {
    this._gaugeSprite = new Sprite();
    this._gaugeSprite.anchor.x = 0.0;
    this._gaugeSprite.anchor.y = 0.0;
    this._gaugeSprite.bitmap = new Bitmap(700, 40);
    this._gaugeSprite.bitmap.fillRect(5, 5, 700 - 10, 30, "#000000");
    this._gaugeSprite.bitmap.fillRect(5, 5, 300 - 10, 30, "#ffff00");
    this.addChild(this._gaugeSprite);
  };

  Oex_Sprite_NyoGauge.prototype.createLabelSprite = function () {
    this._labelSprite = new Sprite();
    this._labelSprite.anchor.x = 0.0;
    this._labelSprite.anchor.y = 0.0;
    this._labelSprite.bitmap = new Bitmap(700, 60);
    this.addChild(this._labelSprite);
    //this._labelSprite.bitmap.fontFace = this.labelFontFace();
    this._labelSprite.bitmap.fontSize = 32;
    this._labelSprite.bitmap.textColor = "#ffffff";
    this._labelSprite.bitmap.outlineColor = "#000000";
    this._labelSprite.bitmap.outlineWidth = 4;
    this._labelSprite.bitmap.drawText("", 0, 0, 700, 40, "center");
  };

  Oex_Sprite_NyoGauge.prototype.update = function () {
    Sprite_Clickable.prototype.update.call(this);
    this.visible = $gameSwitches.value(11);
    if ($gameSwitches.value(11)) {
      var scale = Math.floor($gameVariables.value(4) / 10);
      this._gaugeSprite.bitmap.clear();
      this._gaugeSprite.bitmap.fillRect(5, 5, 700 - 10, 30, "#000000");
      if (scale > 7 * 80) {
        scale = 7 * 80;
      }
      if (scale > 10) {
        this._gaugeSprite.bitmap.fillRect(5, 5, scale - 10, 30, "#ffff00");
      }
      this._labelSprite.bitmap.clear();
      this._labelSprite.bitmap.drawText("" + Math.floor(scale / 7) + "%", 0, 0, 700, 40, "center");
    }
  };

  //-----------------------------------------------------------------------------

  var createStandFileName = function (_heroineId, _poseId) {
    if (_poseId == 0) {
      return "";
    }
    switch (Number(_heroineId)) {
      case 1:
        return "h1_0" + _poseId;
      case 2:
        return "h2_0" + _poseId;
      case 3:
        return "h3_0" + _poseId;
      case 4:
        return "h4_0" + _poseId;
      case 5:
        return "h5_0" + _poseId;
      case 6:
        return "h6_0" + _poseId;
    }
    return "";
  };

  var Spriteset_Map_createShadow_org = Spriteset_Map.prototype.createShadow;
  Spriteset_Map.prototype.createShadow = function () {
    Spriteset_Map_createShadow_org.apply(this, arguments);
    if (Number($gameMap.mapId()) == 43) {
      this._fogSprite = new Sprite();
      this._fogSprite.x = 0;
      this._fogSprite.y = 0;
      this._fogSprite.z = 6;
      if ($gameSwitches.value(48)) {
        this._fogSprite.bitmap = ImageManager.loadPicture("dark01");
      } else {
        this._fogSprite.bitmap = ImageManager.loadPicture("dark00");
      }
      this.addChild(this._fogSprite);
    }
    this._sanimationSprite = new OexSprite_SideChar();
    this.addChild(this._sanimationSprite);
    this._standSprite01 = new Sprite();
    this.__standSprite_name01 = "";
    this._standSprite01.anchor.x = 0;
    this._standSprite01.anchor.y = 1;
    this._standSprite01.x = -50;
    this._standSprite01.y = 720;
    this._standSprite01.z = 7;
    this.addChild(this._standSprite01);
    this._standSprite02 = new Sprite();
    this.__standSprite_name02 = "";
    this._standSprite02.anchor.x = 0;
    this._standSprite02.anchor.y = 1;
    this._standSprite02.x = 960 - 300;
    this._standSprite02.y = 720;
    this._standSprite02.z = 7;
    this.addChild(this._standSprite02);
    this.tmp_v11 = $gameVariables.value(11);
    this.tmp_v12 = $gameVariables.value(12);
    this.tmp_v13 = $gameVariables.value(13);
    this.tmp_v14 = $gameVariables.value(14);
    this._nyoGaugeSprite = new Oex_Sprite_NyoGauge();
    this.addChild(this._nyoGaugeSprite);
  };

  var Spriteset_Map_updateShadow_org = Spriteset_Map.prototype.updateShadow;
  Spriteset_Map.prototype.updateShadow = function () {
    Spriteset_Map_updateShadow_org.apply(this, arguments);
    var v11 = $gameVariables.value(11);
    var v12 = $gameVariables.value(12);
    var v13 = $gameVariables.value(13);
    var v14 = $gameVariables.value(14);

    if (this.tmp_v11 != v11 || this.tmp_v12 != v12 || this.tmp_v13 != v13 || this.tmp_v14 != v14) {
      this.tmp_v11 = v11;
      this.tmp_v12 = v12;
      this.tmp_v13 = v13;
      this.tmp_v14 = v14;
      this.__standSprite_name01 = createStandFileName(v11, v12);
      this._standSprite01.bitmap = ImageManager.loadPicture(this.__standSprite_name01);
      this.__standSprite_name02 = createStandFileName(v13, v14);
      this._standSprite02.bitmap = ImageManager.loadPicture(this.__standSprite_name02);
      DataManager.addSystemdInfo(this.__standSprite_name01);
      DataManager.addSystemdInfo(this.__standSprite_name02);
    }
  };

  Sprite_Balloon.prototype.speed = function () {
    return 8;
  };

  Sprite_AnimationMV.prototype.setupRate = function () {
    this._rate = 4;
  };

  Game_Temp.prototype.requestBalloon = function (target, balloonId) {
    const request = { target: target, balloonId: balloonId };
    this._balloonQueue.push(request);
    if (target.startBalloon) {
      if (Number(balloonId) == 1) {
        OexUtils.playSound("2k-electronic_sound08");
      } else if (Number(balloonId) != 10) {
        OexUtils.playSound("__button64");
      }
      target.startBalloon();
    }
  };

  //----------------------------------------------------------------------

  TouchInput._createNewState = function () {
    return {
      triggered: false,
      cancelled: false,
      moved: false,
      hovered: false,
      released: false,
      middled: false,
      wheelX: 0,
      wheelY: 0,
    };
  };

  TouchInput._onMiddleButtonDown = function (event) {
    const x = Graphics.pageToCanvasX(event.pageX);
    const y = Graphics.pageToCanvasY(event.pageY);
    if (Graphics.isInsideCanvas(x, y)) {
      this._onMiddle(x, y);
    }
  };

  TouchInput._onMiddle = function (x, y) {
    this._newState.middled = true;
    this._x = x;
    this._y = y;
  };

  var TouchInput_update_org = TouchInput.update;
  TouchInput.update = function () {
    this._middled = this._newState.middled;
    this._newState.middled = false;
    TouchInput_update_org.apply(this, arguments);
  };

  TouchInput.isMiddled = function () {
    return this._middled;
  };

  //---------------------------------------------------------------

  Game_Map.prototype.parallaxName = function () {
    if ($dataMap.meta.indream == "true") {
      return "_dream00";
    }
    return this._parallaxName;
  };

  Game_Map.prototype.updateParallax = function () {
    if ($dataMap.meta.indream == "true") {
      this._parallaxSx = 2;
      this._parallaxSy = 2;
      this._parallaxLoopX = true;
      this._parallaxLoopY = true;
    }
    if (this._parallaxLoopX) {
      this._parallaxX += this._parallaxSx / this.tileWidth() / 2;
    }
    if (this._parallaxLoopY) {
      this._parallaxY += this._parallaxSy / this.tileHeight() / 2;
    }
  };

  //---------------------------------------------------------------

  Window_Base.prototype.convertEscapeCharacters = function (text) {
    /* eslint no-control-regex: 0 */
    text = text.replace(/\\/g, "\x1b");
    text = text.replace(/\x1b\x1b/g, "\\");
    while (text.match(/\x1bV\[(\d+)\]/gi)) {
      text = text.replace(/\x1bV\[(\d+)\]/gi, (_, p1) => $gameVariables.value(parseInt(p1)));
    }
    text = text.replace(/\x1bFN\[(\d+)\]/gi, (_, p1) => this.colorWrap(14, this.actorFullName(parseInt(p1))));
    text = text.replace(/\x1bN\[(\d+)\]/gi, (_, p1) => this.colorWrap(14, this.actorName(parseInt(p1))));
    text = text.replace(/\x1bJ\[(\d+)\]/gi, (_, p1) => this.colorWrap(14, this.itemName(parseInt(p1))));
    text = text.replace(/\x1bP\[(\d+)\]/gi, (_, p1) => this.partyMemberName(parseInt(p1)));
    text = text.replace(/\x1bG/gi, TextManager.currencyUnit);
    return text;
  };

  Window_Base.prototype.colorWrap = function (n, str) {
    return "\x1b" + "c[" + n + "]" + str + "\x1b" + "c[0]";
  };

  Window_Base.prototype.itemName = function (n) {
    var item = n >= 1 ? $dataItems[n] : null;
    return item ? item.name : "";
  };

  Window_Base.prototype.actorFullName = function (n) {
    const actor = n >= 1 ? $gameActors.actor(n) : null;
    return actor ? actor.name() + " " + actor.nickname() : "";
  };

  //---------------------------------------------------------------

  var Game_Map_createVehicles_org = Game_Map.prototype.createVehicles;
  Game_Map.prototype.createVehicles = function () {
    Game_Map_createVehicles_org.apply(this, arguments);
    this._footPrints = [];
  };

  var Game_Map_refereshVehicles_org = Game_Map.prototype.refereshVehicles;
  Game_Map.prototype.refereshVehicles = function () {
    Game_Map_refereshVehicles_org.apply(this, arguments);
    this._footPrints = [];
  };

  var Game_Map_updateVehicles_org = Game_Map.prototype.updateVehicles;
  Game_Map.prototype.updateVehicles = function () {
    Game_Map_refereshVehicles_org.apply(this, arguments);
    const x = $gamePlayer.x;
    const y = $gamePlayer.y;
    var d = -1;
    var tag = $gamePlayer.terrainTag();
    if (this._footPrints.length > 0) {
      if (this._footPrints[0][0] == x && this._footPrints[0][1] == y) {
      } else {
        if (Number(tag) == 5) {
          d = this.calcFootPrintDirection(this._footPrints[0][0], this._footPrints[0][1], x, y);
        }
        this._footPrints[0][2] = d;
        this._footPrints.unshift([x, y, d]);
        if (this.checkCircle()) {
          $gameSwitches.setValue(46, true);
        }
      }
    } else {
      this._footPrints.unshift([x, y, d]);
    }
    if (this._footPrints.length > 12) {
      this._footPrints.pop();
    }
    // console.log(this._footPrints);
  };

  Game_Map.prototype.calcFootPrintDirection = function (px, py, cx, cy) {
    var n1 = 0;
    if (cx < px) {
      n1 = 0;
    } else if (cx == px) {
      n1 = 1;
    } else if (cx > px) {
      n1 = 2;
    }
    var n2 = 0;
    if (cy < py) {
      n2 = 0;
    } else if (cy == py) {
      n2 = 1;
    } else if (cy > py) {
      n2 = 2;
    }
    var _array = [
      [18, 8, 16],
      [4, 0, 6],
      [12, 2, 14],
    ];
    return _array[n2][n1];
  };

  Game_Map.prototype.checkCircle = function () {
    if (Number(this._mapId) != 41) {
      return false;
    }
    if (!$gameSwitches.value(45)) {
      return false;
    }
    if ($gameSwitches.value(46)) {
      return false;
    }
    if (this._footPrints.length == 0) {
      return false;
    }
    const px = $gamePlayer.x;
    const py = $gamePlayer.y;
    var fp = this._footPrints[this._footPrints.length - 1];
    if (Math.abs(fp[0] - px) <= 1 && Math.abs(fp[1] - py) <= 1) {
    } else {
      return false;
    }
    var geometry = [];
    const mapWidth = this.width();
    const mapHeight = this.height();
    for (var x = 0; x < mapWidth; x++) {
      var arry = [];
      for (var y = 0; y < mapHeight; y++) {
        arry.push(-1);
      }
      geometry.push(arry);
    }
    for (const fp0 of this._footPrints) {
      geometry[fp0[0]][fp0[1]] = 2;
    }
    var candidates = [[0, 0]];
    var new_candidates = [];
    while (candidates.length > 0) {
      for (const pos of candidates) {
        if (0 <= pos[0] && pos[0] < mapWidth && 0 <= pos[1] && pos[1] < mapHeight) {
          if (geometry[pos[0]][pos[1]] == -1) {
            geometry[pos[0]][pos[1]] = 1;
            new_candidates.push([pos[0] - 1, pos[1]]);
            new_candidates.push([pos[0] + 1, pos[1]]);
            new_candidates.push([pos[0], pos[1] - 1]);
            new_candidates.push([pos[0], pos[1] + 1]);
          }
        }
      }
      candidates = new_candidates;
      new_candidates = [];
    }
    var count = 0;
    for (var x = 0; x < mapWidth; x++) {
      for (var y = 0; y < mapHeight; y++) {
        if (geometry[x][y] == -1) {
          count += 1;
        }
      }
    }
    // console.log("count:" + count);
    return count >= 5;
  };

  var Spriteset_Map_createCharacters_org = Spriteset_Map.prototype.createCharacters;
  Spriteset_Map.prototype.createCharacters = function () {
    this._footPrintSprites = [];
    for (var i = 0; i < 12; i++) {
      var sp = new Sprite();
      sp.z = 1;
      sp.bitmap = ImageManager.loadPicture("footPrint");
      sp.visible = false;
      this._footPrintSprites.push(sp);
      this._tilemap.addChild(sp);
    }
    Spriteset_Map_createCharacters_org.apply(this, arguments);
  };

  var Spriteset_Map_updateTileset_org = Spriteset_Map.prototype.updateTileset;
  Spriteset_Map.prototype.updateTileset = function () {
    Spriteset_Map_updateTileset_org.apply(this, arguments);
    for (var i = 0; i < 12; i++) {
      var sp = this._footPrintSprites[i];
      if ($gameMap._footPrints.length > i) {
        var fp = $gameMap._footPrints[i];
        if (fp[2] > 0) {
          const tw = $gameMap.tileWidth();
          sp.x = Math.floor($gameMap.adjustX(fp[0]) * tw);
          const th = $gameMap.tileHeight();
          sp.y = Math.floor($gameMap.adjustY(fp[1]) * th);
          if (fp[2] < 10) {
            sp.setFrame(0, 48 * (fp[2] / 2 - 1), 48, 48);
          } else {
            sp.setFrame(48, 48 * ((fp[2] - 10) / 2 - 1), 48, 48);
          }
          sp.visible = true;
          if (i == 0) {
            sp.visible = false;
          }
        } else {
          sp.visible = false;
        }
      } else {
        sp.visible = false;
      }
    }
  };

  Window_NameInput.prototype.table = function () {
    return [Window_NameInput.LATIN1];
  };

  //---------------------------------------------------------------

  var Game_Event_update_org = Game_Event.prototype.update;
  Game_Event.prototype.update = function () {
    Game_Event_update_org.apply(this, arguments);
    var eventdata = this.event();
    if (eventdata) {
      if (eventdata.meta) {
        if (eventdata.meta.opacity) {
          this._opacity = Number(eventdata.meta.opacity);
        }
      }
    }
  };

  //---------------------------------------------------------------

  Game_Player.prototype.refresh = function () {
    const characterName = "_heroines00";
    const actor = $gameParty.leader();
    var characterIndex = actor ? actor.characterIndex() : 0;
    if ($gameSwitches.value(48)) {
      characterIndex = 1;
    }
    this.setImage(characterName, characterIndex);
    this._followers.refresh();
  };
})();
